/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Adblock for Mozilla.
 *
 * The Initial Developer of the Original Code is
 * Henrik Aasted Sorensen.
 * Portions created by the Initial Developer are Copyright (C) 2002
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * Henrik Aasted Sorensen
 * Stefan Kinitz
 * Wladimir Palant
 * rue
 * ***** END LICENSE BLOCK ***** */

var listBox;
var urlList;


//start-finish functions

function init() {
    var Branch = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("adblock.");
    var hideGlobal = Branch.prefHasUserValue("hideglobal") ? Branch.getBoolPref("hideglobal") : false;
    var hideEnabled = Branch.prefHasUserValue("hideenabled") ? Branch.getBoolPref("hideenabled") : true;
    urlList = Branch.prefHasUserValue("hidelist") ? Branch.getCharPref("hidelist") : "";
    
    document.getElementById("hide-global").setAttribute("checked",hideGlobal);
    document.getElementById("hide-enabled").setAttribute("checked",hideEnabled);
    listBox  = document.getElementById("listBox");
    populateListBox();
    urlCheck();
    toggleGlobalAccess();
}

function save() {
    var Branch = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("adblock.");
    var listSort = Branch.prefHasUserValue("listsort") ? Branch.getBoolPref("listsort") : true;
    
    var hideGlobal = document.getElementById("hide-global");
	Branch.setBoolPref("hideglobal", (hideGlobal.getAttribute("checked") == "true") ? true:false);
    
    var hideEnabled = document.getElementById("hide-enabled");
	Branch.setBoolPref("hideenabled", (hideEnabled.getAttribute("checked") == "true") ? true:false);
    
    //sort list before saving
    if (listSort) {
        var sites = url2array(urlList);
        sites.sort();
        urlList = array2url(sites);
    }
    
    Branch.setCharPref("hidelist", urlList);
    
    window.close();
}


//listbox functions

function populateListBox() {
    var sites = url2array(urlList);
    
    //remove existing list
    for (var i=listBox.getRowCount()-1; i>=0; i--)
        listBox.removeItemAt(i);
    
    //append list
    for (var j=0; j<sites.length; j++)
        listBox.appendItem(sites[j],sites[j]);
    
}

function urlChanged() {
    var urlText = document.getElementById("urlText");
    var url = urlText.value;
    var sites = url2array(urlList);
    var match = false;
    
    //prevent spaces
    if(url.match(/\s/)) url = urlText.value=url.replace(/\s/g,'');
    
    for (var i=0; i<sites.length && sites!=""; i++)
        if (url.indexOf(sites[i]) != -1) match = true;
    
    if (url.length > 0 && !match)
        document.getElementById("addButton").setAttribute("disabled",false);
    else
        document.getElementById("addButton").setAttribute("disabled",true);
}

function addSite() {
    var urlText = document.getElementById("urlText");
    var site = urlText.value;
    urlText.value = "";
    document.getElementById("addButton").setAttribute("disabled",true);
    
    
    if (urlList.length == 0) urlList = site;
    else urlList += " " + site;
    
    focusTextBox();
    populateListBox();
    listBox.ensureIndexIsVisible(listBox.getRowCount()-1);
}

function getCurrentSite() {
    var site = window.opener.opener.content.location.href;
    
    // Don't get domain if not an http site
    if (/^https?:\/\//.test(site) == false)
        return null;

    // Extract domain
    var dom = /\/\/[^\/]+\//;
    site = dom.exec(site)[0];
    
    // Remove matched '/' and 'www.'
    if (/www\./.test(site))
        site = site.slice(6,site.length-1);
    else
        site = site.slice(2,site.length-1);

    return site;
}

function addCurrentSite() {
    
    var site = getCurrentSite();
    
    focusTextBox();
    if (site == null) return;

    // Add to hide url list
    if (urlList.length == 0) urlList = site;
    else urlList += " " + site;
    
    populateListBox();
    listBox.ensureIndexIsVisible(listBox.getRowCount()-1);
    document.getElementById("addCurrentButton").setAttribute("disabled",true);
}

function urlCheck() {
    var url = getCurrentSite();
    var sites = url2array(urlList);
    var match;
    
    for (var i=0; i<sites.length; i++)
        if (url == sites[i]) match = true;
        
    if (!match)
        document.getElementById("addCurrentButton").setAttribute("disabled",false);
    else
        document.getElementById("addCurrentButton").setAttribute("disabled",true);
}

function toggleGlobalAccess() {
    var checked = (document.getElementById("hide-enabled").getAttribute("checked") == "true") ? true:false;
    
    if (checked)
        document.getElementById("hide-global").setAttribute("disabled",false);
    else
        document.getElementById("hide-global").setAttribute("disabled",true);
}

function removeAll() {
    urlList = "";
    populateListBox();
    focusTextBox();
    document.getElementById("addCurrentButton").setAttribute("disabled",false);
}

function remove() {
    const selectedItems = listBox.selectedItems;
    var sites = urlList.split(" ");
    var match = false;
    urlList = "";
    
    for (var i=0; i<sites.length; i++) {
    
        for(var j=0; j<selectedItems.length; j++)
            if (sites[i] == selectedItems[j].value) match = true;
        
        if (!match) {
            if (urlList.length == 0) urlList = sites[i];
            else urlList += " " + sites[i];
        }
        else
            match = false;
    }
    
    populateListBox();
    urlCheck();
}

function focusTextBox()
{
    document.getElementById("urlText").focus();
}


//IO functions

function importSites() {
	var fp = Components.classes["@mozilla.org/filepicker;1"].createInstance(Components.interfaces.nsIFilePicker);
	var stream = Components.classes["@mozilla.org/network/file-input-stream;1"].createInstance(Components.interfaces.nsIFileInputStream);
	var streamIO = Components.classes["@mozilla.org/scriptableinputstream;1"].createInstance(Components.interfaces.nsIScriptableInputStream);
	var Branch = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("adblock.");
    var listSort = !Branch.prefHasUserValue("listsort") || Branch.getBoolPref("listsort"); // default:true
    var overwriteCurrentList = false;
	var input;
	var inputArray;
	var validFile = false;
	
	fp.init(window, document.getElementById("bundle_adblock").getString("adblock.selectafile"), fp.modeOpen);
	fp.appendFilters(fp.filterText);

	if (fp.show() != fp.returnCancel) {
		stream.init(fp.file, 0x01, 0444, null);
		streamIO.init(stream);
		input = streamIO.read(stream.available());
		streamIO.close();
		stream.close();
		
		linebreak = input.match(/(?:\[[Aa]d[Bb]lock\s[Hh]ide\s[Ll]ist\])(((\n+)|(\r+))+)/m)[1]; // first: whole match -- second: backref-1 -- etc..
		inputArray = input.split(linebreak);
		
		var headerRe = /\[[Aa]d[Bb]lock\s[Hh]ide\s[Ll]ist\]/; // tests if the first line is adblock's header
		if (headerRe.test(inputArray[0])) {
			inputArray.shift();
			overwriteCurrentList = confirm(document.getElementById("bundle_adblock").getString("adblock.confirmoverwrite"));
            
            //sort imported site list
            if (listSort) inputArray.sort();
            
            //append list removing duplicates
            if (!overwriteCurrentList) {
                
                var existingList = url2array(urlList);
                var match = false;
            
                for (var i=0; i<inputArray.length; i++) {
            
                    for (var j=0; j<existingList.length; j++)
                        if (inputArray[i] == existingList[j]) match = true;
                    
                    if (!match) {
                        if (!/^[\s]*$/.test(inputArray[i])) urlList += " " + inputArray[i];
                    }
                    else
                        match = false;
                }
            }
            else //replace existing list
                urlList = array2url(inputArray);

        }
		else 
			alert(document.getElementById("bundle_adblock").getString("adblock.filenotvalid"));
	}
    populateListBox();
}

function exportSites() {
	var fp = Components.classes["@mozilla.org/filepicker;1"].createInstance(Components.interfaces.nsIFilePicker);
	var stream = Components.classes["@mozilla.org/network/file-output-stream;1"].createInstance(Components.interfaces.nsIFileOutputStream);

	fp.init(window, document.getElementById("bundle_adblock").getString("adblock.selectafile"), fp.modeSave);
	fp.appendFilters(fp.filterText);
    fp.defaultExtension=".txt";

	if (fp.show() != fp.returnCancel) {
		if (fp.file.exists())
			fp.file.remove(true);
		fp.file.create(fp.file.NORMAL_FILE_TYPE, 0666);

		stream.init(fp.file, 0x02, 0x200, null);
		stream.write("[Adblock Hide List]\n", 20);
		
        var sites = url2array(urlList);
		
 		for (var i = 0; i<sites.length; i++) {
			stream.write(sites[i], sites[i].length);
			if (i < (sites.length - 1))
				stream.write("\n", 1);
		}
		stream.close();
	}
}


//utility functions

function url2array(url)
{
    return url.split(" ");
}

function array2url(array)
{
    var url = array[0];
    
    for (var i=1; i<array.length; i++)
        url += " " + array[i];
    
    return url;
}

